#!/usr/bin/python
# -*- coding: <UTF-8> -*-

"""
Python Version : 3.5

$LastChangedRevision: 6277 $
$LastChangedDate:: 2022-10-20 07:22:41 +0200#$

This file is part of SyMSpace/ComponentSpace.

Copyright (c) 2018, Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
All rights reserved.
"""

import os
import shutil
import json
from symspace.wrapper.symwrapper import SymProject, SymContainer, SymField, SymDirectory, \
    SymFile, SymMaterial, SymPythonFunction
from componentspace.base_component import Component

try:
    # noinspection PyUnresolvedReferences
    from creo.jLink import JLinkInterface
    # noinspection PyUnresolvedReferences
    from magopt.utils import MopFile, ComponentInfo
except Exception as e:
    print(e)


class DXFGeometryComponent(Component):
    """
    Create geometry component definition project
    """

    # ToDo: create a __init__ method which is compatible with the superclass Component
    def __init__(self, comp_file):
        super().__init__(comp_file)
        self.__base_directory = self.path

    @property
    def base_directory(self):
        return self.__base_directory

    @base_directory.setter
    def base_directory(self, val: str):
        self.__base_directory = val

    def preBuildTest(self) -> dict:
        """
        Get parameters for test file

        Returns
        -------
        d : dict
        """
        mop_file = os.path.join(os.path.dirname(self.path), self.path, self.name)

        if not os.path.isfile(mop_file) and not os.path.isfile(mop_file + ".mop"):
            raise FileNotFoundError(mop_file + " can not be found.")
        sym_mdl = SymProject.load(mop_file)
        if not isinstance(sym_mdl, SymProject):
            raise Exception(mop_file + " can not be loaded.")

        webpath = sym_mdl['Info']['WebPath'].getData()
        comp_webpath = '.'.join([webpath, self.name])

        testparam = sym_mdl['Testparameter']

        sym_mdl.close()

        if testparam is None:
            raise Exception('Container Testparameter does not exist!')

        test_mop_file = os.path.join(self.base_directory, 'tests', 'test_{}'.format(self.name))

        d = {'MopFile': test_mop_file,
             'WebPath': comp_webpath,
             'TestParam': testparam}
        return d

    def generateHelpFiles(self, comp_cont: SymContainer):
        """
        Generate help files

        Help files are copied to the directory above

        Parameters
        ----------
        comp_cont : SymContainer
        """
        super().generateHelpFiles(comp_cont)

        rdir = self.path.split('\\')[-1]
        sdir = os.path.join(self.path, 'rst')
        ddir = sdir.replace('\\' + rdir, '')
        # print(ddir)
        try:
            sfile = os.path.join(sdir, 'Param.txt')
            if os.path.isfile(sfile):
                shutil.copy(sfile, ddir)
        except:
            print('Cannot copy Param.txt')
        try:
            sfile = os.path.join(sdir, 'Paramdetail.txt')
            if os.path.isfile(sfile):
                shutil.copy(sfile, ddir)
        except:
            print('Cannot copy Paramdetail.txt')
        try:
            sfile = os.path.join(sdir, 'ParamIn.txt')
            if os.path.isfile(sfile):
                shutil.copy(sfile, ddir)
        except:
            print('Cannot copy ParamIn.txt')
        try:
            sfile = os.path.join(sdir, 'ParamOut.txt')
            if os.path.isfile(sfile):
                shutil.copy(sfile, ddir)
        except:
            print('Cannot copy ParamOut.txt')

    def writeUnitFile(self):
        """
        Writes Unit file for GeometryPart function
        """
        units = dict()
        file = None
        for inparam in self.parCAD.getImports():
            par = inparam.getHandle()
            if par is not None:
                if hasattr(par, 'getUnit') and par.getUnit() != '':
                    units[par.getName()] = par.getUnit()
                if inparam.getVarName() == 'Part':
                    file = par.getData()
        if file is not None:
            ext = os.path.splitext(file)[1]
            jsonfile = file.replace(ext, '.json')
            with open(jsonfile, 'w') as fp:
                json.dump({'cad': self.geo_class, 'units': units}, fp)
