#!/usr/bin/python
# -*- coding: <UTF-8> -*-

"""
Python Version : 3.5

$LastChangedRevision: 7858 $
$LastChangedDate:: 2024-02-27 10:49:30 +0100#$

This file is part of SyMSpace.

Copyright (c) 2017, Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
All rights reserved.
"""
import numpy as np
from at.lcm.bu21.general.dtypes.unit import UnitConversion

from symspace.wrapper.symwrapper import SymContainer, SymField


class GeneralMaterialFunctions:

    @staticmethod
    def set_filter(prj, material_dict):
        if 'usergroup' in material_dict:
            prj['Filter.UserGroup'].setData(material_dict['usergroup'])

    @staticmethod
    def set_info(prj, material_dict):
        if 'info_name' in material_dict:
            prj['Info.Name'].setData(material_dict['info_name'])
        else:
            prj['Info.Name'].setData(material_dict['component_name'])

        if 'info_functionname' in material_dict:
            prj['Info.FunctionName'].setData(material_dict['info_functionname'])
        else:
            prj['Info.FunctionName'].setData(material_dict['component_name'])

        if 'info_displayname' in material_dict:
            prj['Info.DisplayName'].setData(material_dict['info_displayname'])
        else:
            prj['Info.DisplayName'].setData(material_dict['component_name'].replace('_', ' '))

        if 'webpath' in material_dict:
            prj['Info.WebPath'].setData(material_dict['webpath'])

    @staticmethod
    def set_general_properties(prj, material_dict):

        prj['Project.Material.Type'].setData(material_dict['class'])
        prj['Project.Material.Name'].setData(material_dict['name'])
        prj['Project.Material.Manufacturer'].setData(material_dict['manufacturer'])
        GeneralMaterialFunctions.insert_magnet_class(prj, material_dict)

        GeneralMaterialFunctions.insert_value_string(prj, material_dict, 'DIN EN', 'Project.Material.DIN-Norm')
        GeneralMaterialFunctions.insert_unit_data(prj, material_dict, 'Price', 'Project.Material.price', "Euro")
        GeneralMaterialFunctions.insert_unit_data(prj, material_dict, 'Density', 'Project.Material.rho', "kg.dm-3")
        GeneralMaterialFunctions.insert_unit_data(prj, material_dict, 'Lamination thickness', 'Project.Material.tlam', "mm")

        if 'Stacking factor' in material_dict and GeneralMaterialFunctions.is_float(material_dict['Stacking factor']['value']):
            prj['Project.Material.ks'].setData(float(material_dict['Stacking factor']['value']))
        elif 'Lamination thickness' in material_dict and GeneralMaterialFunctions.is_float(material_dict['Lamination thickness']['value']):
            GeneralMaterialFunctions.insert_stacking_factor(prj, material_dict, float(material_dict['Lamination thickness']['value']), 'Project.Material.ks')
        else:
            prj['Project.Material.ks'].removeParam()
            print('Lamination thickness not defined for ' + material_dict['component_name'])

        print('Reference temperature')
        GeneralMaterialFunctions.insert_unit_data(prj, material_dict, 'Reference temperature', 'Project.Material.Theta', "degC", "{:.1f}", 20.0)
        print('Reference temperature set')

        prj['Project.Material.colorR'].setData(int(material_dict['ColorR']['value']))
        prj['Project.Material.colorG'].setData(int(material_dict['ColorG']['value']))
        prj['Project.Material.colorB'].setData(int(material_dict['ColorB']['value']))

        GeneralMaterialFunctions.insert_unit_data(prj, material_dict, 'Kinematic viscosity', 'Project.Material.nu_visc', "m2.s-1")
        GeneralMaterialFunctions.insert_unit_data(prj, material_dict, 'Dynamic viscosity', 'Project.Material.eta_visc', "Pa.s", "{:.3g}")

    @staticmethod
    def set_electromagnetic_properties(prj, material_dict):

        if 'Resistivity' in material_dict and GeneralMaterialFunctions.is_float(material_dict['Resistivity']['value']):
            GeneralMaterialFunctions.insert_unit_data(prj, material_dict, 'Resistivity', 'Project.Material.rhoel', "Ohm.mm2.m-1", "{:.4f}")  # given in [Ohm.mm2.m-1]
        elif 'Specific electrical resistance' in material_dict and GeneralMaterialFunctions.is_float(material_dict['Specific electrical resistance']['value']):
            GeneralMaterialFunctions.insert_unit_data(prj, material_dict, 'Specific electrical resistance', 'Project.Material.rhoel', "Ohm.mm2.m-1", "{:.4f}")  # given in 1e2 [micro.Ohm.cm] = 1 [Ohm.mm2.m-1]
        else:
            prj['Project.Material.rhoel'].removeParam()
            print('Resistivity' + ' not defined for ' + material_dict['component_name'])

        GeneralMaterialFunctions.insert_conductivity(prj, material_dict)

        GeneralMaterialFunctions.insert_unit_data(prj, material_dict, 'khfe', 'Project.Material.khfe', "W.kg-1")
        GeneralMaterialFunctions.insert_unit_data(prj, material_dict, 'kefe', 'Project.Material.kefe', "W.kg-1")

        prj['Project.Material.kefe_F'].removeParam()  # [W.kg-1]
        prj['Project.Material.khfe_F'].removeParam()  # [W.kg-1]

        if 'H' in material_dict and 'B' in material_dict:
            prj['Project.Material.HvsB.H'].setData(np.array(material_dict['H']))  # unlock, lock?
            prj['Project.Material.HvsB.B'].setData(np.array(material_dict['B']))  # unlock, lock?
        else:
            prj['Project.Material.HvsB'].removeParam()

        GeneralMaterialFunctions.insert_unit_data(prj, material_dict, 'Remanence flux density', 'Project.Material.Br', "T", "{:.3f}")
        GeneralMaterialFunctions.insert_unit_data(prj, material_dict, 'Relative permeability', 'Project.Material.mur', "1")

        if 'HvsTheta' in material_dict:
            prj['Project.Material.HvsTheta.Theta'].setData(np.array(material_dict['HvsTheta']['Theta']['value']))
            prj['Project.Material.HvsTheta.Hcb'].setData(np.array(material_dict['HvsTheta']['Hcb']['value']))
        else:
            prj['Project.Material.HvsTheta'].removeParam()

    @staticmethod
    def set_mechanic_properties(prj, material_dict):
        if 'Hardness HV5 (VPN)' in material_dict:
            GeneralMaterialFunctions.insert_unit_data(prj, material_dict, 'Hardness HV5 (VPN)', 'Project.Material.HV', "1")
        elif 'Hardness' in material_dict:
            GeneralMaterialFunctions.insert_unit_data(prj, material_dict, 'Hardness', 'Project.Material.HV', "1")
        else:
            prj['Project.Material.HV'].removeParam()
            print('Hardness not defined for ' + material_dict['component_name'])

        GeneralMaterialFunctions.insert_unit_data(prj, material_dict, 'Shore Hardness HAD', 'Project.Material.HAD', "1")
        GeneralMaterialFunctions.insert_unit_data(prj, material_dict, 'Tensile strength', 'Project.Material.Rm', "N.mm-2")  # 1 [MPa] = 1 [N.mm-2]
        if '0.2%-Yield strength' in material_dict:
            GeneralMaterialFunctions.insert_unit_data(prj, material_dict, '0.2%-Yield strength', 'Project.Material.Rp02', "N.mm-2")  # 1 [MPa] = 1 [N.mm-2]
        elif '0.2 %-Yield strength' in material_dict:
            GeneralMaterialFunctions.insert_unit_data(prj, material_dict, '0.2 %-Yield strength', 'Project.Material.Rp02', "N.mm-2")  # 1 [MPa] = 1 [N.mm-2]
        else:
            prj['Project.Material.Rp02'].removeParam()
            print('0.2%-Yield strength not defined for ' + material_dict['component_name'])
        GeneralMaterialFunctions.insert_unit_data(prj, material_dict, 'Poisson ratio', 'Project.Material.nu', "1")
        GeneralMaterialFunctions.insert_young_modulus(prj, material_dict)
        GeneralMaterialFunctions.insert_tangent_modulus(prj, material_dict)
        GeneralMaterialFunctions.insert_stress_strain_curve(prj, material_dict)

    @staticmethod
    def set_thermal_properties(prj, material_dict):
        GeneralMaterialFunctions.insert_unit_data(prj, material_dict, 'Min temperature', 'Project.Material.Tmin', "degC")
        GeneralMaterialFunctions.insert_unit_data(prj, material_dict, 'Max temperature', 'Project.Material.Tmax', "degC")

        GeneralMaterialFunctions.insert_unit_data(prj, material_dict, 'Specific thermal capacity', 'Project.Material.Cm', "J.kg-1.K-1")
        GeneralMaterialFunctions.insert_unit_data(prj, material_dict, 'Temperature coefficient specific thermal capacity', 'Project.Material.alpha_CmT', "K-1")
        GeneralMaterialFunctions.insert_unit_data(prj, material_dict, 'Thermal conductivity', 'Project.Material.lambda', "W.m-1.K-1")  # 1 [W.m-1.K-1]

        GeneralMaterialFunctions.insert_unit_data(prj, material_dict, 'Temperature coefficient thermal conductivity', 'Project.Material.alpha_lambdaT', "K-1")
        GeneralMaterialFunctions.insert_exact_unit_data(prj, material_dict, 'Temperature coefficient ohmic resistance', 'Project.Material.alpha_RT', "K-1")

        GeneralMaterialFunctions.insert_exact_unit_data(prj, material_dict, 'Temperature coefficient volume expansion', 'Project.Material.alpha_VT', "K-1")
        GeneralMaterialFunctions.insert_unit_data(prj, material_dict, 'Temperature coefficient remanence flux density', 'Project.Material.alpha_BrT', "K-1", "{:.6f}")

    @staticmethod
    def insert_conductivity(prj, dict):
        if 'Resistivity' in dict and GeneralMaterialFunctions.is_float(dict['Resistivity']['value']):
            uc = UnitConversion(dict['Resistivity']['unit'].replace('[', '').replace(']', ''), "Ohm.mm2.m-1")
            value = 1.0 / (uc.convert(float(dict['Resistivity']['value'])))  # 1/ [Ohm.mm2.m-1] = S.m.mm-2
            prj['Project.Material.kappa_20'].setData(float("{:.6g}".format(value)))
        elif 'Specific electrical resistance' in dict and GeneralMaterialFunctions.is_float(dict['Specific electrical resistance']['value']):
            uc = UnitConversion(dict['Specific electrical resistance']['unit'].replace('[', '').replace(']', ''), "Ohm.mm2.m-1")
            value = 1.0 / (uc.convert(float(dict['Specific electrical resistance']['value'])))  # 1/ [Ohm.mm2.m-1] = S.m.mm-2
            prj['Project.Material.kappa_20'].setData(float("{:.6g}".format(value)))
        else:
            prj['Project.Material.kappa_20'].removeParam()
            print('Resistivity' + ' not defined for ' + dict['component_name'])

    @staticmethod
    def insert_magnet_class(prj, dict):
        magnet_types = ['AlNiCo', 'AlNiCo sintered', 'AlNiCo cast', 'Ferrite', 'Ferrite isotropic',
                        'Ferrite anisotropic', 'Ferrite injection molded', 'NdFeB', 'NdFeB bonded', 'NdFeB extruded',
                        'NdFeB injection molded', 'SmCo', 'SmCo injection molded', 'SmFeN injection molded']

        if 'Werkstoff' in dict:
            temp = dict['Werkstoff']
            if temp in magnet_types:
                prj['Project.Material.Class'].setData(temp)
            elif temp == 'Sm2Co17' or temp == 'SmCo5':
                prj['Project.Material.Class'].setData('SmCo')
            else:
                print(temp + " no considered magnet class.")
                prj['Project.Material.Class'].removeParam()
        else:
            print("Werkstoff not defined in material dictionary.")
            prj['Project.Material.Class'].removeParam()

    @staticmethod
    def insert_young_modulus(prj, dict):
        if 'Young\u2019s modulus (RD)' in dict and 'Young\u2019s modulus (TD)' in dict:
            uc = UnitConversion(dict['Young\u2019s modulus (RD)']['unit'].replace('[', '').replace(']', ''), "kN.mm-2")
            modulus = uc.convert(min(float(dict['Young\u2019s modulus (RD)']['value'].replace(" ", "")), float(
                dict['Young\u2019s modulus (TD)']['value'].replace(" ", ""))))
            prj['Project.Material.E'].setData(modulus)
        elif 'Young modulus' in dict:
            uc = UnitConversion(dict['Young modulus']['unit'].replace('[', '').replace(']', ''), "kN.mm-2")
            modulus = uc.convert(float(dict['Young modulus']['value']))
            prj['Project.Material.E'].setData(modulus)
        else:
            prj['Project.Material.E'].removeParam()
            print('Young modulus not defined for ' + dict['component_name'])

    @staticmethod
    def insert_tangent_modulus(prj, dict):
        if 'Tangent modulus' in dict:
            uc = UnitConversion(dict['Tangent modulus']['unit'].replace('[', '').replace(']', ''), "N.mm-2")
            modulus = uc.convert(float(dict['Tangent modulus']['value']))
            prj['Project.Material.E_t'].setData(modulus)
        else:
            prj['Project.Material.E_t'].removeParam()
            print('Tangent modulus not defined for ' + dict['component_name'])

    @staticmethod
    def insert_stress_strain_curve(prj, dict):
        if 'StressStrainCurve' in dict:
            # # check if the container 'StressStrainCurve' exists; if not, add it to the project
            # has_stress_strain_curve = False
            # for i in range(len(prj['Project.Material'].getParam())):
            #     if prj['Project.Material'].getParam()[i].getName() == 'StressStrainCurve':
            #         has_stress_strain_curve = True
            #         break
            #
            # if not has_stress_strain_curve:
            #     prj['Project.Material'].addParam(SymContainer('StressStrainCurve'))

            for key in dict['StressStrainCurve']:
                container_name = str(key).replace('.', '_')
                container = SymContainer(container_name)
                prj['Project.Material.StressStrainCurve'].addParam(container)
                stress_array = SymField('Stress')
                prj['Project.Material.StressStrainCurve.' + str(container_name)].addParam(stress_array)
                prj['Project.Material.StressStrainCurve.' + str(container_name) + '.Stress'].setData(np.array(dict['StressStrainCurve'][key]['Stress']['value']))

                strain_array = SymField('PlasticStrain')
                prj['Project.Material.StressStrainCurve.' + str(container_name)].addParam(strain_array)
                prj['Project.Material.StressStrainCurve.' + str(container_name) + '.PlasticStrain'].setData(np.array(dict['StressStrainCurve'][key]['PlasticStrain']['value']))
        else:
            # # check if the container 'StressStrainCurve' exists; if yes, remove it from the project
            # for i in range(len(prj['Project.Material'].getParam())):
            #     if prj['Project.Material'].getParam()[i].getName() == 'StressStrainCurve':
            #         prj['Project.Material.StressStrainCurve'].removeParam()
            #         break
            prj['Project.Material.StressStrainCurve'].removeParam()

    @staticmethod
    def insert_data(prj, dict, dict_key, material_parameter, uc):

        if dict_key in dict and GeneralMaterialFunctions.is_float(dict[dict_key]):
            # prj[material_parameter].setData(float(dict[dict_key]) * conversion_factor)
            prj[material_parameter].setData(float("{:.2f}".format(uc.convert(float(dict[dict_key])))))
        else:
            prj[material_parameter].removeParam()
            print(dict_key + ' not defined for ' + dict['component_name'])

    @staticmethod
    def insert_data_value(prj, dict, dict_key, material_parameter, uc):

        if dict_key in dict and GeneralMaterialFunctions.is_float(dict[dict_key]['value']):
            # prj[material_parameter].setData(float(dict[dict_key]['value']) * conversion_factor)
            prj[material_parameter].setData(float("{:.2f}".format(uc.convert(float(dict[dict_key]['value'])))))
        else:
            prj[material_parameter].removeParam()
            print(dict_key + ' not defined for ' + dict['component_name'])

    @staticmethod
    def insert_unit_data(prj, dict, dict_key, material_parameter, target_unit, f_digits="{:.2f}", default_value=None):
        print("dict_key: ", dict_key)

        if dict_key in dict and GeneralMaterialFunctions.is_float(dict[dict_key]['value']):
            uc = UnitConversion(dict[dict_key]['unit'].replace('[', '').replace(']', ''), target_unit)
            prj[material_parameter].setData(float(f_digits.format(uc.convert(float(dict[dict_key]['value'])))))
        elif default_value is not None:
            prj[material_parameter].setData(default_value)
            print(material_parameter + 'not defined for ' + dict['component_name'] + ". Set to default = " + str(default_value) + " degC.")
        else:
            prj[material_parameter].removeParam()
            print(dict_key + ' not defined for ' + dict['component_name'])

    @staticmethod
    def insert_exact_unit_data(prj, dict, dict_key, material_parameter, target_unit):
        print("dict_key: ", dict_key)
        if dict_key in dict and GeneralMaterialFunctions.is_float(dict[dict_key]['value']):
            uc = UnitConversion(dict[dict_key]['unit'].replace('[', '').replace(']', ''), target_unit)
            prj[material_parameter].setData(uc.convert(float(dict[dict_key]['value'])))

        else:
            prj[material_parameter].removeParam()
            print(dict_key + ' not defined for ' + dict['component_name'])

    @staticmethod
    def insert_exact_data_value(prj, dict, dict_key, material_parameter, conversion_factor):

        if dict_key in dict and GeneralMaterialFunctions.is_float(dict[dict_key]['value']):
            # prj[material_parameter].setData(float(dict[dict_key]['value']) * conversion_factor)
            prj[material_parameter].setData(float(dict[dict_key]['value']) * conversion_factor)
        else:
            prj[material_parameter].removeParam()
            print(dict_key + ' not defined for ' + dict['component_name'])

    @staticmethod
    def insert_value_string(prj, dict, dict_key, material_parameter):

        if dict_key in dict and GeneralMaterialFunctions.is_float(dict[dict_key]['value']):
            # prj[material_parameter].setData(float(dict[dict_key]['value']) * conversion_factor)
            prj[material_parameter].setData(dict[dict_key]['value'])
        else:
            prj[material_parameter].removeParam()
            print(dict_key + ' not defined for ' + dict['component_name'])

    @staticmethod
    def insert_stacking_factor(prj, dict, tlam, material_parameter):
        # Stacking factor: rule of thumb from https://www.emetor.com/glossary/stacking-factor/
        # for tlam in mm:
        # 0.0425 * tlam + 0.6032 * arctan(108.16 * tlam)
        if abs(float(tlam) - 0.1) < 0.001:
            dict['Stacking factor'] = {'variable': 'ks', 'value': 0.90, 'unit': '1'}
        elif abs(float(tlam) - 0.15) < 0.001:
            dict['Stacking factor'] = {'variable': 'ks', 'value': 0.92, 'unit': '1'}
        elif abs(float(tlam) - 0.2) < 0.001:
            dict['Stacking factor'] = {'variable': 'ks', 'value': 0.93, 'unit': '1'}
        elif abs(float(tlam) - 0.25) < 0.001:
            dict['Stacking factor'] = {'variable': 'ks', 'value': 0.94, 'unit': '1'}
        elif abs(float(tlam) - 0.27) < 0.001:
            dict['Stacking factor'] = {'variable': 'ks', 'value': 0.94, 'unit': '1'}
        elif abs(float(tlam) - 0.3) < 0.001:
            dict['Stacking factor'] = {'variable': 'ks', 'value': 0.94, 'unit': '1'}
        elif abs(float(tlam) - 0.35) < 0.001:
            dict['Stacking factor'] = {'variable': 'ks', 'value': 0.95, 'unit': '1'}
        elif abs(float(tlam) - 0.50) < 0.001:
            dict['Stacking factor'] = {'variable': 'ks', 'value': 0.96, 'unit': '1'}
        elif abs(float(tlam) - 0.65) < 0.001:
            dict['Stacking factor'] = {'variable': 'ks', 'value': 0.97, 'unit': '1'}
        elif abs(float(tlam) - 1.0) < 0.001:
            dict['Stacking factor'] = {'variable': 'ks', 'value': 0.98, 'unit': '1'}
        else:
            print('Stacking factor not defined!')

        dict_key = 'Stacking factor'
        print('ks: ', float(dict[dict_key]['value']))
        if dict_key in dict and GeneralMaterialFunctions.is_float(dict[dict_key]['value']):
            prj[material_parameter].setData(float(dict[dict_key]['value']))
            print('ks in material: ', float(dict[dict_key]['value']))
        else:
            prj[material_parameter].removeParam()
            print(dict_key + ' not defined for ' + dict['component_name'])

    @staticmethod
    def is_float(element):
        return_value = True

        if isinstance(element, list):
            print("Element is a list, not a float")
            return_value = False
        else:
            try:
                float(element)
            except ValueError:
                print("Not a float")
                return_value = False

        return return_value

    @staticmethod
    def interpolate_data(arg, value, arg_ref):

        interpolated_value = None

        if value is not None and arg is not None:
            if arg_ref < np.min(arg) or arg_ref > np.max(arg):
                raise Exception('Reference value outside specification')
            interpolated_value = np.interp(arg_ref, arg.flatten(), value.flatten())

        return interpolated_value

